import { IFunction } from '@aws-cdk/aws-lambda';
import * as cdk from '@aws-cdk/core';
/**
 * @stability stable
 */
export interface LambdaSubminuteProps {
    /**
     * The Lambda function that is going to be executed per time unit less than one minute.
     *
     * @stability stable
     */
    readonly targetFunction: IFunction;
    /**
     * A pattern you want this statemachine to be executed.
     *
     * @default cron(50/1 15-17 ? * * *) UTC+0 being run every minute starting from 15:00 PM to 17:00 PM.
     * @see https://docs.aws.amazon.com/AmazonCloudWatch/latest/events/ScheduledEvents.html
     * @stability stable
     */
    readonly conjobExpression?: string;
    /**
     * How many times you intent to execute in a minute.
     *
     * @default 6
     * @stability stable
     */
    readonly frequency?: number;
    /**
     * Seconds for an interval, the product of `frequency` and `intervalTime` should be approximagely 1 minute.
     *
     * @default 10
     * @stability stable
     */
    readonly intervalTime?: number;
}
/**
 * @stability stable
 */
export declare class LambdaSubminute extends cdk.Construct {
    /**
     * The Lambda function that plays the role of the iterator.
     *
     * @stability stable
     */
    readonly iteratorFunction: IFunction;
    /**
     * The ARN of the state machine that executes the target Lambda function per time unit less than one minute.
     *
     * @stability stable
     */
    readonly stateMachineArn: string;
    /**
     * @stability stable
     */
    constructor(parent: cdk.Construct, name: string, props: LambdaSubminuteProps);
}
